@@ -68,12 +68,13 @@ class ObschartApiClient(object):
             + programTrackActionResponseFragment
         )
 
-        filter = filter.copy()
         now = datetime.datetime.utcnow()
-        filter.update(
-            {"createdAt": {"after": now.replace(tzinfo=datetime.timezone.utc).isoformat()}}
-        )
-        variables = {"filter": filter}
+        variables = {
+            "filter": {
+                **filter,
+                "createdAt": {"after": now.replace(tzinfo=datetime.timezone.utc).isoformat()},
+            }
+        }
 
         while True:
             start = time.monotonic()
@@ -214,9 +215,7 @@ class ObschartApiClient(object):
 
         return ProgramTrackActionResponse(result["programTrackActionResponse"], self._context)
 
-    async def update_program_track_action_response(
-        self, id: str, feedback_program_track_action_id: str
-    ):
+    async def update_program_track_action_response(self, id: str, *, input: Dict[str, Any]):
         query = gql(
             """
             mutation UpdateProgramTrackActionResponseMutation($input: UpdateProgramTrackActionResponseInput!)  {
@@ -230,10 +229,7 @@ class ObschartApiClient(object):
             + programTrackActionResponseFragment
         )
 
-        input = {
-            "id": id,
-            "feedbackProgramTrackActionId": feedback_program_track_action_id,
-        }
+        input = {"id": id, **input}
         result = await self._context.client._execute_mutation(query, input)
 
         return ProgramTrackActionResponse(
@@ -395,3 +391,111 @@ class ObschartApiClient(object):
         result = await self._execute(query, variables)
 
         return Application(result["application"], self._context)
+
+    async def update_program_track_action(self, id: str, *, input: Dict[str, Any]):
+        query = gql(
+            """
+            mutation UpdateProgramTrackActionMutation($input: UpdateProgramTrackActionInput!)  {
+                updateProgramTrackAction(input: $input) {
+                    programTrackAction {
+                        ...ProgramTrackAction
+                    }
+                }
+            }
+        """
+            + programTrackActionFragment
+        )
+
+        input = {"id": id, **input}
+        result = await self._context.client._execute_mutation(query, input)
+
+        return ProgramTrackAction(
+            result["updateProgramTrackAction"]["programTrackAction"], self._execute,
+        )
+
+    async def update_application_request(self, id: str, *, input: Dict[str, Any]):
+        query = gql(
+            """
+            mutation UpdateApplicationRequestMutation($input: UpdateApplicationRequestInput!)  {
+                updateApplicationRequest(input: $input) {
+                    applicationRequest {
+                        ...ApplicationRequest
+                    }
+                }
+            }
+        """
+            + applicationRequestFragment
+        )
+
+        input = {"id": id, **input}
+        result = await self._context.client._execute_mutation(query, input)
+
+        return ApplicationRequest(
+            result["updateApplicationRequest"]["applicationRequest"], self._execute,
+        )
+
+    async def poll_application_requests(self, filter: dict):
+        query = gql(
+            """
+          query PollApplicationRequestsQuery($filter: ApplicationRequestFilterInput) {
+            applicationRequests(filter: $filter, first: 9999) {
+              edges {
+                node {
+                  ...ApplicationRequest
+                }
+              }
+            }
+          }
+        """
+            + applicationRequestFragment
+        )
+
+        now = datetime.datetime.utcnow()
+        variables = {
+            "filter": {
+                **filter,
+                "createdAt": {"after": now.replace(tzinfo=datetime.timezone.utc).isoformat()},
+            }
+        }
+
+        while True:
+            start = time.monotonic()
+            try:
+                response = await self._execute(query, variables, timeout=1)
+                edges = response["applicationRequests"]["edges"]
+
+                if len(edges) > 0:
+                    break
+            except httpx.TimeoutException as exception:
+                log.warn(f"Polling timed out: {exception.__class__.__name__}")
+            except:
+                log.exception("Polling failed")
+
+            duration = time.monotonic() - start
+            await asyncio.sleep(max(1 - duration, 0))
+
+        requests: List[ApplicationRequest] = [
+            ApplicationRequest(edge["node"], self._context) for edge in edges
+        ]
+
+        return requests
+
+    async def create_program_track_action_response(self, *, input: Dict[str, Any]):
+        query = gql(
+            """
+          mutation CreateProgramTrackActionResponseMutation($input: CreateProgramTrackActionResponseInput!) {
+            createProgramTrackActionResponse(input: $input) {
+              programTrackActionResponse  {
+                ...ProgramTrackActionResponse
+              }
+            }
+          }
+          """
+            + programTrackActionResponseFragment
+        )
+
+        result = await self._execute_mutation(query, input)
+
+        return ProgramTrackActionResponse(
+            result["createProgramTrackActionResponse"]["programTrackActionResponse"], self._context
+        )
