@@ -1,4 +1,5 @@
 import io
+from typing import Dict, List
 
 
 def is_extractable_file(thing):
@@ -7,7 +8,7 @@ def is_extractable_file(thing):
 
 def extract_files(value, path=""):
     clone = None
-    files: dict[io.BufferedReader, list[str]] = dict()
+    files: Dict[io.BufferedReader, List[str]] = dict()
 
     def add_file(paths, file):
         storedPaths = files.get(file, [])
